package main

import (
        "bytes"
	"C"
	"fmt"
        "os"
	"unsafe"
        "net/http"
        "crypto/tls"
        "strings"
	"github.com/fluent/fluent-bit-go/output"
)

//export FLBPluginRegister
func FLBPluginRegister(def unsafe.Pointer) int {
	return output.FLBPluginRegister(def, "fluentbit_wendelin", "Fluentbit output plugin for wendelin")
}

//export FLBPluginInit
// (fluentbit will call this)
// plugin (context) pointer to fluentbit context (state/ c code)
func FLBPluginInit(plugin unsafe.Pointer) int {
	streamtool_uri := output.FLBPluginConfigKey(plugin, "streamtool_uri")
        key := output.FLBPluginConfigKey(plugin, "key")
        certificate := output.FLBPluginConfigKey(plugin, "certificate")
        buffer_type := output.FLBPluginConfigKey(plugin, "buffer_type")
        flush_interval := output.FLBPluginConfigKey(plugin, "flush_interval")
        disable_retry_limit := output.FLBPluginConfigKey(plugin, "disable_retry_limit")
	reference := output.FLBPluginConfigKey(plugin, "reference")
        dict := map[string]string{
          "streamtool_uri": streamtool_uri,
          "key": key,
          "certificate": certificate,
          "buffer_type": buffer_type,
          "flush_interval": flush_interval,
          "disable_retry_limit": disable_retry_limit,
	  "reference": reference,
        }
        output.FLBPluginSetContext(plugin, dict)
	return output.FLB_OK
}

//export FLBPluginFlushCtx
func FLBPluginFlushCtx(ctx, data unsafe.Pointer, length C.int, tag *C.char) int {
	var ret int
	var record map[interface{}]interface{}
	// Create Fluent Bit decoder
	dec := output.NewDecoder(data, int(length))
        dict := output.FLBPluginGetContext(ctx).(map[string]string)
	// Iterate Records
        var result string
        result = ""
        var is_end bool = false
	for {
		// Extract Record
		ret, _, record = output.GetRecord(dec)
		if ret != 0 {
			break
		}

		// Print record keys and values
		for _, v := range record {
                   var output_string string = ""
                   for _, s := range v.([]uint8) {
                     output_string  = output_string + string(s)
                   }
                   if strings.Contains(output_string, "fluentbit_end") {
                     is_end = true
                   }
                   result = result + output_string 
		}
                result += "\n"
	}

	// Return options:
	//
	// output.FLB_OK    = data have been processed.
	// output.FLB_ERROR = unrecoverable error, do not try this again.
	// output.FLB_RETRY = retry to flush later.
        //body result
        // content type  "application/octet-stream"
        var b = []byte(result)
        cert, err := tls.LoadX509KeyPair(dict["certificate"], dict["key"])
	if err != nil {
          fmt.Fprintf(os.Stderr, "server: loadkey %s", err.Error())
	}
        uri := fmt.Sprintf("%s/uploadComputerFileMetadata?reference=%s", dict["streamtool_uri"], dict["reference"])
        client := &http.Client{
          Transport: &http.Transport{
            TLSClientConfig: &tls.Config{
              InsecureSkipVerify: true,
              Certificates: []tls.Certificate{cert},
           },
          },
        }
        req, err := http.NewRequest("POST", uri, bytes.NewReader(b))
        if err != nil {
          fmt.Fprintf(os.Stderr, "Got error %s", err.Error())
          return output.FLB_RETRY
        }
        req.Header.Set("Content-Type", "application/octet-stream")
        rsp, err := client.Do(req)
        if err != nil {
          fmt.Fprintf(os.Stderr, "got error %s", err.Error())
          return output.FLB_RETRY
        }
        if rsp.StatusCode != 204 {
          fmt.Fprintf(os.Stderr, "status code %d", rsp.StatusCode)
          return output.FLB_RETRY
        }
        if is_end {
          os.Exit(0)
        }
        return output.FLB_OK
}

//export FLBPluginExit
func FLBPluginExit() int {
	return output.FLB_OK
}

func main() {
}
